#pragma once

#include <unordered_map>
#include <sstream>

/**
 * Helper class to parse command line arguments.
 * Supports arguments in a the following form:
 *   --argument-name=argument-value
 * or
 *   --argument-name
 */
class Arguments
{
public:
    Arguments(int argc, char** argv);

    /**
     * Does the named argument exist?
     */
    bool hasArgument(const std::string& name);

    /**
     * Get the value of an argument.
     * Empty if no value was specified.
     */
    template <typename T>
    T value(const std::string& name);

private:
    std::unordered_map<std::string, std::string> m_arguments;
};

template <typename T>
T Arguments::value(const std::string& name)
{
    if (!hasArgument(name)) {
        return {};
    }

    std::stringstream ss(m_arguments[name]);

    T value;
    ss >> value;
    return value;
}
