#pragma once

#include <vector>
#include <Windows.h>
#include <GL/glew.h>
#include <Varjo_gl.h>

#include "IRenderer.hpp"
#include "RenderTarget.hpp"

class GLRenderer : public IRenderer
{
public:
    GLRenderer(varjo_Session* session, bool useMultiTexture);
    ~GLRenderer();

    std::shared_ptr<Geometry> createGeometry(uint32_t vertexCount, uint32_t indexCount) override;
    std::shared_ptr<RenderTarget> createRenderTarget(int32_t width, int32_t height, varjo_TextureFormat format) override;
    std::shared_ptr<RenderTarget> createRenderTarget(int32_t width, int32_t height, varjo_Texture texture) override;

private:
    bool initVarjo() override;

    void useRenderTarget(const std::shared_ptr<RenderTarget>& renderTarget) override;
    void clearRenderTarget(const std::shared_ptr<RenderTarget>& renderTarget, float r, float g, float b, float a) override;

    void useGeometry(const std::shared_ptr<Geometry>& geometry) override;

    void setupCamera(const glm::mat4& viewMatrix, const glm::mat4& projectionMatrix) override;
    void setViewport(int32_t x, int32_t y, int32_t width, int32_t height) override;

    void drawGrid() override;
    void queueObjects(const std::vector<glm::mat4>& matrices) override;
    void dispatchDraws() override;
    void advance() override;

    void compileShaders();
    void compileShader();
    void compileGridShader();

    void createPerFrameBuffers();
    GLuint createInstanceBuffer();
    GLuint createUniformBuffer();
    void useInstanceBuffer();

    /**
     * Creates a dummy window.
     * This is requred for the OpenGL context.
     */
    void createDummyWindow();

    struct ShaderUniforms {
        glm::mat4 viewMatrix;
        glm::mat4 projectionMatrix;
    };

    struct InstanceBuffer {
        int32_t maxInstances;
        int32_t queued;

        glm::mat4* data;
    };

    struct PerFrameBuffers {
        GLuint uniformBuffer;
        GLuint instanceBuffer;
    };

    int32_t m_currentFrameBuffer = 0;
    std::vector<PerFrameBuffers> m_perFrameBuffers;

    InstanceBuffer m_instanceBuffer = {};
    ShaderUniforms m_shaderUniforms;

    HWND m_hwnd = 0;
    HDC m_hdc = 0;
    HGLRC m_hglrc = 0;

    GLuint m_gridProgram = 0;
    GLuint m_program = 0;
};
