#pragma once

#define WIN32_LEAN_AND_MEAN
#include <Windows.h>
#include <fstream>
#include <vector>
#include <string>
#include <chrono>

class Profiler
{
public:
    void addSample()
    {
        if (!m_started) {
            m_startTime = getCurrentTime();
            m_started = true;
        } else {
            double endTime = getCurrentTime();
            double elapsed = endTime - m_startTime;
            m_startTime = endTime;

            m_frameTimes.push_back(elapsed);
        }
    }

    int32_t sampleCount() const { return static_cast<int32_t>(m_frameTimes.size()); }

    void exportCSV(const std::string& fileName)
    {
        std::ofstream file(fileName);
        if (!file.is_open()) {
            printf("Profiler::exportCSV: Failed to open %s\n", fileName.c_str());
            return;
        }

        size_t frameCount = m_frameTimes.size();
        for (size_t i = 0; i < frameCount; ++i) {
            file << i + 1 << "," << m_frameTimes[i] << "\n";
        }
    }

private:
    double getCurrentTime()
    {
        using namespace std::chrono;
        auto ns = duration_cast<nanoseconds>(system_clock::now().time_since_epoch());
        return static_cast<double>(ns.count()) * 0.000001;
    }

    bool m_started = false;
    double m_startTime;

    std::vector<double> m_frameTimes;
};
