#pragma once

#include <GL/glew.h>
#include <d3d11.h>
#include <Varjo_d3d11.h>
#include <Varjo_gl.h>

class D3D11Renderer;

class RenderTarget
{
public:
    RenderTarget(int32_t width, int32_t height);
    virtual ~RenderTarget();

    // Convert the render target to a varjo_Texture that can be passed to Varjo API.
    virtual varjo_Texture toVarjoTexture() = 0;

    int32_t width() const { return m_width; }
    int32_t height() const { return m_height; }

protected:
    int32_t m_width;
    int32_t m_height;
};

class D3D11RenderTarget : public RenderTarget
{
public:
    D3D11RenderTarget(D3D11Renderer* renderer, int32_t width, int32_t height, ID3D11Texture2D* backBuffer);
    D3D11RenderTarget(D3D11Renderer* renderer, int32_t width, int32_t height, varjo_TextureFormat format);
    ~D3D11RenderTarget();

    // Convert the render target to a varjo_Texture that can be passed to Varjo API.
    varjo_Texture toVarjoTexture() override;

    ID3D11Texture2D* backBuffer() const { return m_backBuffer; }
    ID3D11RenderTargetView* renderTargetView() const { return m_renderTargetView; }
    ID3D11DepthStencilView* depthStencilView() const { return m_depthStencilView; }

private:
    void create(D3D11Renderer* renderer, ID3D11Texture2D* backBuffer);

    ID3D11DepthStencilView* m_depthStencilView;
    ID3D11RenderTargetView* m_renderTargetView;
    ID3D11Texture2D* m_backBuffer;
};

class GLRenderTarget : public RenderTarget
{
public:
    GLRenderTarget(int32_t width, int32_t height, GLuint texture);
    GLRenderTarget(int32_t width, int32_t height, varjo_TextureFormat format);
    ~GLRenderTarget();

    // Convert the render target to a varjo_Texture that can be passed to Varjo API.
    varjo_Texture toVarjoTexture() override;

    GLuint backBuffer() const { return m_colorBuffer; }
    GLuint frameBuffer() const { return m_frameBuffer; }

private:
    void create(GLuint colorBuffer);

    GLuint m_frameBuffer;
    GLuint m_depthBuffer;
    GLuint m_colorBuffer;
};
